'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var _defineProperty = _interopDefault(require('@babel/runtime/helpers/defineProperty'));
var _classCallCheck = _interopDefault(require('@babel/runtime/helpers/classCallCheck'));
var _createClass = _interopDefault(require('@babel/runtime/helpers/createClass'));
var _possibleConstructorReturn = _interopDefault(require('@babel/runtime/helpers/possibleConstructorReturn'));
var _getPrototypeOf = _interopDefault(require('@babel/runtime/helpers/getPrototypeOf'));
var _inherits = _interopDefault(require('@babel/runtime/helpers/inherits'));
var React = _interopDefault(require('react'));
var reactDnd = require('react-dnd');
var cx = _interopDefault(require('classnames'));
var _toConsumableArray = _interopDefault(require('@babel/runtime/helpers/toConsumableArray'));
var _objectWithoutProperties = _interopDefault(require('@babel/runtime/helpers/objectWithoutProperties'));

var TYPE = 'TreeNode';

var TreeViewInsertTarget =
/*#__PURE__*/
function (_React$Component) {
  _inherits(TreeViewInsertTarget, _React$Component);

  function TreeViewInsertTarget() {
    _classCallCheck(this, TreeViewInsertTarget);

    return _possibleConstructorReturn(this, _getPrototypeOf(TreeViewInsertTarget).apply(this, arguments));
  }

  _createClass(TreeViewInsertTarget, [{
    key: "render",
    value: function render() {
      var _this$props = this.props,
          connectDropTarget = _this$props.connectDropTarget,
          insertBefore = _this$props.insertBefore,
          canDrop = _this$props.canDrop,
          isDropping = _this$props.isDropping,
          classNames = _this$props.classNames;
      return connectDropTarget(React.createElement("div", {
        className: cx(insertBefore ? classNames.insertBeforeTarget : classNames.insertAfterTarget, _defineProperty({}, classNames.insertTargetCanDrop || 'insertTargetCanDrop', canDrop), _defineProperty({}, classNames.insertTargetDropping || 'insertTargetDropping', isDropping))
      }, React.createElement("div", {
        className: cx(_defineProperty({}, classNames.insertTargetMarkerDropping, isDropping))
      })));
    }
  }]);

  return TreeViewInsertTarget;
}(React.Component);

var handleCanDrop = function handleCanDrop(props, monitor, item) {
  return (// block dropping if a prosp.node.noDrop === true
    (!props.parentNode || props.parentNode && !props.parentNode.noDrop) && // cannot drop to self
    !(props.parentNode === item.parentNode && (props.parentChildIndex === item.parentChildIndex || props.parentChildIndex === item.parentChildIndex + 1)) && // you cannot drop on self nodes
    !item.allSourceIDs.includes(props.parentNode ? props.parentNode.id : null)
  );
};

var handleDrop = function handleDrop(props, monitor, component, item) {
  props.onMoveNode({
    oldParentNode: item.parentNode,
    oldParentChildIndex: item.parentChildIndex,
    oldPrecedingNode: item.precedingNode,
    node: item.node,
    newParentNode: props.parentNode,
    newParentChildIndex: props.parentChildIndex,
    newPrecedingNode: props.precedingNode
  });
  return {
    parentNode: props.parentNode,
    parentChildIndex: props.parentChildIndex
  };
};

var nodeTarget = {
  drop: function drop(props, monitor, component) {
    return monitor.didDrop() ? undefined : // some child already handled drop
    handleDrop(props, monitor, component, monitor.getItem());
  },
  canDrop: function canDrop(props, monitor) {
    return handleCanDrop(props, monitor, monitor.getItem());
  }
};

var collectNodeDropProps = function collectNodeDropProps(connect, monitor) {
  return {
    connectDropTarget: connect.dropTarget(),
    canDrop: monitor.canDrop(),
    isDropping: monitor.isOver({
      shallow: true
    }) && monitor.canDrop()
  };
};

var DroppedTarget = reactDnd.DropTarget([TYPE], nodeTarget, collectNodeDropProps);
var DroppableTreeViewInsertTarget = DroppedTarget(TreeViewInsertTarget);

var TreeViewItem =
/*#__PURE__*/
function (_React$Component) {
  _inherits(TreeViewItem, _React$Component);

  function TreeViewItem() {
    _classCallCheck(this, TreeViewItem);

    return _possibleConstructorReturn(this, _getPrototypeOf(TreeViewItem).apply(this, arguments));
  }

  _createClass(TreeViewItem, [{
    key: "render",
    value: function render() {
      var _this$props = this.props,
          connectDragSource = _this$props.connectDragSource,
          props = _objectWithoutProperties(_this$props, ["connectDragSource"]);

      return connectDragSource && connectDragSource(React.createElement("div", {
        className: cx(props.classNames.node || 'node', _defineProperty({}, props.classNames.nodeDragging || 'nodeDragging', props.isDragging)),
        key: props.node.id
      }, props.node.children ? React.createElement(DroppableTreeViewItemNode, {
        parentNode: props.node,
        parentChildIndex: 0,
        precedingNode: null,
        onMoveNode: props.onMoveNode,
        renderNode: props.renderNode,
        classNames: props.classNames
      }) : React.createElement("div", null, props.renderNode(props.node)), !props.node.isCollapsed && React.createElement("div", {
        className: props.classNames.nodeChildren
      }, props.node.children && props.node.children.length > 0 && React.createElement(TreeViewItemList, {
        parentNode: props.node,
        nodes: props.node.children ? props.node.children : [],
        classNames: props.classNames,
        renderNode: props.renderNode,
        onMoveNode: props.onMoveNode
      }))));
    }
  }]);

  return TreeViewItem;
}(React.Component);

var DroppableTreeViewItemNode = DroppedTarget(
/*#__PURE__*/
function (_React$Component2) {
  _inherits(_class, _React$Component2);

  function _class() {
    _classCallCheck(this, _class);

    return _possibleConstructorReturn(this, _getPrototypeOf(_class).apply(this, arguments));
  }

  _createClass(_class, [{
    key: "render",
    value: function render() {
      var props = this.props;
      return props.connectDropTarget(React.createElement("div", {
        className: props.classNames && props.isDropping ? props.classNames.insertNodeDropping : null
      }, props.renderNode(props.parentNode)));
    }
  }]);

  return _class;
}(React.Component)); // append current node as well as children's node using flatMap

var gatherNodeIDs = function gatherNodeIDs(node) {
  var _ref;

  return [node.id].concat(_toConsumableArray((_ref = []).concat.apply(_ref, _toConsumableArray((node.children || []).map(gatherNodeIDs)))));
};

var nodeSource = {
  beginDrag: function beginDrag(props, monitor, component) {
    return {
      node: props.node,
      allSourceIDs: gatherNodeIDs(props.node),
      parentNode: props.parentNode,
      parentChildIndex: props.parentChildIndex,
      precedingNode: props.precedingNode
    };
  },
  canDrag: function canDrag(props, monitor) {
    return !props.node.locked;
  }
};

var collectNodeDragProps = function collectNodeDragProps(connect, monitor) {
  return {
    connectDragSource: connect.dragSource(),
    isDragging: monitor.isDragging()
  };
};

var DraggableTreeViewItem = reactDnd.DragSource(TYPE, nodeSource, collectNodeDragProps)(TreeViewItem);

var nodesWithPredecessors = function nodesWithPredecessors(nodes) {
  return nodes.map(function (node) {
    return {
      node: node,
      precedingNode: node
    };
  });
}; // TODO: add a mechanism to apply the CSS equivalent:
// .nodePositioningWrapper:hover {
//   /* otherwise drop targets interfere with drag start */
//   z-index: 2;
// }


function TreeViewItemList(props) {
  var withPredecessors = nodesWithPredecessors(props.nodes);
  return React.createElement("div", {
    className: props.classNames.nodeList
  }, withPredecessors.map(function (node, index) {
    return React.createElement("div", {
      key: node.node.id,
      style: {
        position: 'relative'
      },
      className: props.classNames.nodePositioningWrapper
    }, index === 0 ? React.createElement(DroppableTreeViewInsertTarget, {
      classNames: props.classNames,
      insertBefore: true,
      parentNode: props.parentNode,
      parentChildIndex: index,
      precedingNode: null,
      onMoveNode: props.onMoveNode
    }) : null, React.createElement(DroppableTreeViewInsertTarget, {
      classNames: props.classNames,
      insertBefore: false,
      parentNode: props.parentNode,
      parentChildIndex: index + 1,
      precedingNode: node.node,
      onMoveNode: props.onMoveNode
    }), React.createElement(DraggableTreeViewItem, {
      parentNode: props.parentNode,
      parentChildIndex: index,
      precedingNode: node.precedingNode,
      node: node.node,
      classNames: props.classNames,
      renderNode: props.renderNode,
      onMoveNode: props.onMoveNode
    }));
  }));
}

var TreeView =
/*#__PURE__*/
function (_React$Component) {
  _inherits(TreeView, _React$Component);

  function TreeView() {
    _classCallCheck(this, TreeView);

    return _possibleConstructorReturn(this, _getPrototypeOf(TreeView).apply(this, arguments));
  }

  _createClass(TreeView, [{
    key: "render",
    value: function render() {
      var _this$props = this.props,
          classNames = _this$props.classNames,
          innerRef = _this$props.innerRef,
          props = _objectWithoutProperties(_this$props, ["classNames", "innerRef"]);

      return React.createElement("div", {
        ref: innerRef,
        className: classNames.treeView
      }, React.createElement(TreeViewItemList, {
        lock: props.lock,
        parentNode: null,
        nodes: props.rootNodes,
        renderNode: props.renderNode,
        classNames: classNames,
        onMoveNode: props.onMoveNode
      }));
    }
  }]);

  return TreeView;
}(React.Component);

var TreeView$1 = React.forwardRef(function (props, ref) {
  return React.createElement(TreeView, Object.assign({}, props, {
    innerRef: ref
  }));
});

module.exports = TreeView$1;
